// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 10 enero 2012
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY 
//
// PCCM ParrillaCocinaCircularMetal
// El archivo dibuja una parrilla negra simple.
// Puede utilizar la siguiente macro sin parametros
// #macro PCCM_Parrilla()
//
// Genera una parrilla con las siguientes caracteristicas
// Minimo : < -76,   0, -76 >*L
// Maximo : < 174,  23,  76 >*L
// Las patas sobre el plano XZ, y la agarradera hacia X+
//
// Un poco mas flexible es la siguiente macro
//  #macro PCCM_ParrillaCocinaCircularMetal(
//                DiametroParrilla, DiametroAlambre, AlturaParrilla,
//                TexturaBorde, TexturaParrillaRed,
//                TexturaAgarradera, TexturaPatitas )
//      DiametroParrilla        : Diametro del circulo para comida.
//              Default 146*L.
//      DiametroAlambre         : Grueso del alambre de soporte.
//              Default 2.15*L.
//      AlturaParrilla          : Referencia para la altura de parrilla.
//              En realidad el dibujo es algo mayor.  Default 20*L.
//      TexturaBorde            : Se aplica al borde circular, y la
//              linea interna que prolonga la agarradera.
//      TexturaParrillaRed      : Textura a aplicar a la rejilla
//              metalica.
//      TexturaAgarradera       : Textura a aplicar a la agarradera.
//      TexturaPatitas          : Textura a aplicar a las patitas.
//
// Las siguientes variables no deberian modificarse
// PCCM_TexturaBorde            :
//      Se aplica al borde circular, y la
//      linea interna que prolonga la agarradera.
// PCCM_TexturaParrillaRed      :
//      Textura a aplicar a la rejilla metalica.
// PCCM_TexturaAgarradera       :
//      Textura a aplicar a la agarradera.
// PCCM_TexturaPatitas          :
//      Textura a aplicar a las patitas.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)
// #declare PCCM_POV3_1         = off;
//      Valor on activa modo de compatibilidad con POVRay 3.1.
//      En algunas versiones se activara automaticamente.
// #declare PCCM_DefaultTexturaBorde            = texture...
//      Textura default que se aplica al borde circular, y la
//      linea interna que prolonga la agarradera.
// #declare PCCM_DefaultTexturaParrillaRed      = texture...
//      Textura default a aplicar a la rejilla metalica.
// #declare PCCM_DefaultTexturaAgarradera       = texture...
//      Textura default a aplicar a la agarradera.
// #declare PCCM_DefaultTexturaPatitas          = texture...
//      Textura default a aplicar a las patitas.
//
// Para colocarlo simplemente
// #include "PCCM.inc"
// object { PCCM_Parrilla() }
//
// Otro ejemplo con modificador
// #include "PCCM.inc"
// #declare "PCCM_DefaultTexturaBorde       = texture       {
//   average
//   texture_map        {
//     [ 0.8 Rust                               ]
//     [ 0.2 PCCM_TexturaBorde                  ]
//   }
// }
// object       {
//   PCCM_Parrilla()
// }
//
// Un ejemplo con la macro larga
// #include "PCCM.inc"
// object                {
//    PCCM_ParrillaCocinaCircularMetal( 146*L, 2.15*L, 20*L,
//        PinkAlabaster, PinkAlabaster,
//        PinkAlabaster, PinkAlabaster )
// }
//

#include "colors.inc"

#if ( version < 3.2 )
  #ifndef( PCCM_POV3_1 )
    #declare PCCM_POV3_1        = on;
  #end
#else
  #ifndef( PCCM_POV3_1 )
    #declare PCCM_POV3_1        = off;
  #end
#end

#declare PCCM_L         = 0.001;

#declare PCCM_TexturaBorde              = texture       {
  pigment       { Gray10 }
}

#declare PCCM_TexturaParrillaRed        = texture       {
  pigment       { Gray10 }
}

#declare PCCM_TexturaAgarradera         = texture       {
  pigment       { Gray10 }
}

#declare PCCM_TexturaPatitas            = texture       {
  pigment       { Gray10 }
}

#ifndef ( PCCM_DefaultTexturaBorde )
  #declare PCCM_DefaultTexturaBorde             = texture       {
    PCCM_TexturaBorde
  }
#end

#ifndef ( PCCM_DefaultTexturaPatitas )
  #declare PCCM_DefaultTexturaPatitas           = texture       {
    PCCM_TexturaPatitas
  }
#end

#ifndef ( PCCM_DefaultTexturaParrillaRed )
  #declare PCCM_DefaultTexturaParrillaRed       = texture       {
    PCCM_TexturaParrillaRed
  }
#end

#ifndef ( PCCM_DefaultTexturaAgarradera )
  #declare PCCM_DefaultTexturaAgarradera        = texture       {
    PCCM_TexturaAgarradera
  }
#end

#macro PCCM_ParrillaCocinaCircularMetal(
                DiametroParrilla, DiametroAlambre, AlturaParrilla,
                TexturaBorde, TexturaParrillaRed,
                TexturaAgarradera, TexturaPatitas )
  #local DiametroAlambreRed             = 1.5/2.15*DiametroAlambre;
  #local AnguloSoporte                  = 14;
  #local FraccionReferenciaAgarradera   = 0.6;
  #macro PCCM_PuntoGradoXZ( AnguloGrados )
    DiametroParrilla/2*
    < cos( radians( AnguloGrados ) ), 0, sin( radians( AnguloGrados ) ) >
  #end
  union                 {
    union               {
      torus               {
        DiametroParrilla/2      , DiametroAlambre/2
      }
      cylinder          {
        PCCM_PuntoGradoXZ( AnguloSoporte ),
        PCCM_PuntoGradoXZ( 180-AnguloSoporte ),
        DiametroAlambre/2
        translate       DiametroAlambre*y
      }
      cylinder          {
        PCCM_PuntoGradoXZ( -AnguloSoporte ),
        PCCM_PuntoGradoXZ( -180+AnguloSoporte ),
        DiametroAlambre/2
        translate       DiametroAlambre*y
      }
      texture             {
        TexturaBorde
      }
    }
    union               {
      cylinder          {
        PCCM_PuntoGradoXZ( AnguloSoporte ),
        PCCM_PuntoGradoXZ( AnguloSoporte )
        +DiametroParrilla*FraccionReferenciaAgarradera*x,
        DiametroAlambre/2
        translate       DiametroAlambre*y
      }
      cylinder          {
        PCCM_PuntoGradoXZ( -AnguloSoporte ),
        PCCM_PuntoGradoXZ( -AnguloSoporte )
        +DiametroParrilla*FraccionReferenciaAgarradera*x,
        DiametroAlambre/2
        translate       DiametroAlambre*y
      }
      difference        {
        torus             {
          DiametroParrilla/2*sin( radians( AnguloSoporte) ),
          DiametroAlambre/2
        }
        plane           {
          x, 0
        }
        translate       DiametroAlambre*y+
                        ( DiametroParrilla/2*cos( radians( AnguloSoporte ) )+
                        DiametroParrilla*FraccionReferenciaAgarradera )*x
      }
      texture           {
        TexturaAgarradera
      }
    }
    difference          {
      union             {
        #local C        = 0;
        #while ( C < 4 )
          torus             {
            DiametroParrilla        ,
            DiametroAlambreRed/2
            translate       DiametroParrilla*x+
                          ( 0.35*DiametroParrilla )*x
            rotate      90*y*C
            #if ( mod( C, 2 ) = 1 )
              translate DiametroAlambreRed*y
            #end
          }
          #if ( mod( C, 2 ) = 0 )
            torus             {
              DiametroParrilla        ,
              DiametroAlambreRed/2
              translate       DiametroParrilla*x+
                            ( 0.15*DiametroParrilla )*x
              rotate      90*y*C
            }
          #end
          #local C              = C+1;
        #end
      }
      sphere            {
        0               , ( DiametroParrilla-DiametroAlambre )/2
        inverse
      } 
      texture           {
        TexturaParrillaRed
      }
    }
    #local C            = 0;
    #while ( C < 3 )
      difference        {
        union           {
          torus           {
            AlturaParrilla,
            DiametroAlambre/2
            rotate        83*z
            scale         <1,1,1/3>
          }
          torus           {
            AlturaParrilla,
            DiametroAlambre/2
            rotate        83*z
            scale         <1,1,0.85*1/3>
          }
          torus           {
            AlturaParrilla,
            DiametroAlambre/2
            rotate        83*z
            scale         <1,1,1.15*1/3>
          }
        }
        plane           {
          -y            , 0
        }
        translate       -DiametroParrilla/2*x+DiametroAlambre/2*x
          +AlturaParrilla/3*sin( 2*AlturaParrilla/( 3*DiametroParrilla ) )*x
        rotate          120*C*y
        texture         {
          TexturaPatitas
        }
      }
      #local            C = C+1;
    #end
    translate   AlturaParrilla*y
  }
#end

#macro PCCM_Parrilla()
  #local L                              = PCCM_L;
  #local DiametroParrilla               = 146*L;
  #local DiametroAlambre                = 2.15*L;
  #local AlturaParrilla                 = 20*L;
  object                {
    PCCM_ParrillaCocinaCircularMetal(
        DiametroParrilla, DiametroAlambre, AlturaParrilla,
        PCCM_DefaultTexturaBorde, PCCM_DefaultTexturaParrillaRed,
        PCCM_DefaultTexturaAgarradera, PCCM_DefaultTexturaPatitas )
  }
#end

