// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 9 mayo 2011
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY 
//
// La macro RSBA_RuedaBicicletaPequena()
// genera un objeto con
// las siguientes caracteristicas
// Rueda en el plano xz, centrada en el origen
// Radio Principal      : 215*L
// Diametro Neumatico   : 36*L
// Ancho Total Z        : 93*L
// Ancho Envarillado Z  : 54*L
//
// El objeto puede generarse via una macro principal de texturas
// #macro RBSA_RuedaBicicletaSinAccesorios
//    ( RadioTotal, DiametroRueda, AnchoMetalico, TexturaRodante,
//      TexturaRodante, TexturaRuedaInterior, TexturaMetalica, MaterialLuz )
//      RadioTotal                      : Radio total de rueda
//      DiametroRueda                   : Diametro del caucho
//      AnchoMetalico                   : Ancho en Z del envarillado
//              sin los tornillos y el soporte cilindrico
//      TexturaRodante                  : Textura del caucho oscuro
//              en contacto con la carretera
//      TexturaRuedaInterior            : Textura del caucho hacia
//              el interior de la llanta
//      TexturaMetalica                 : Textura de las varillas
//      MaterialLuz                     : Material de la luz de seguridad
//
// Las siguientes variables no deberian modificarse
// RBSA_TexturaRodante
//      Define la textura del caucho en contacto con la carretera
// RBSA_TexturaRuedaInterior
//      Define la textura del caucho hacia el interior de la llanta
// RBSA_TexturaMetalica
//      Define la textura del envarillado
// RBSA_MaterialLuz
//      Define el material la luz de seguridad
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)
// #declare RBSA_POV3_1         = off;
//      Valor on activa modo de compatibilidad con POVRay 3.1.
//      En algunas versiones se activara automaticamente.
// #declare RBSA_DefaultTexturaRodante                  = texture ...
//      Define la textura utilizada en el caucho en contacto
//      con la carretera
// #declare RBSA_DefaultTexturaRuedaInterior            = texture ...
//      Define la textura utilizada en el caucho hacia el
//      interior de la llanta
// #declare RBSA_DefaultTexturaMetalica                 = textura ...
//      Define la textura utilizada en el envarillado
// #declare RBSA_DefaultMaterialLuz                     = material ...
//      Define el material utilizado en la luz de seguridad
// #declare RBSA_LuzBicicleta                           = on
//      Si se declara off, no se dibuja la luz de seguridad
//
// Para colocarlo simplemente
// #include "RBSA.inc"
// object { RBSA_RuedaBicicletaPequena() }
//
// Otro ejemplo con modificador
// #include "RBSA.inc"
// #declare RBSA_DefaultTexturaMetalica         = texture       {
//   average
//   texture_map        {
//     [ 0.2 Rust                               ]
//     [ 0.8 RBSA_TexturaMetalica               ]
//   }
// }
// object       { RBSA_RuedaBicicletaPequena() }
//

#include "colors.inc"  
#include "textures.inc" 
#include "metals.inc"

#if ( version < 3.2 )
  #ifndef( RBSA_POV3_1 )
    #declare RBSA_POV3_1        = on;
  #end
#else
  #ifndef( RBSA_POV3_1 )
    #declare RBSA_POV3_1        = off;
  #end
#end

#ifndef ( RBSA_LuzBicicleta )
  #declare RBSA_LuzBicicleta    = on;
#end

#declare RBSA_L                 = 0.001;    


#declare RBSA_TexturaRodante            = texture       {  
  #if ( RBSA_POV3_1 )
    #local MyTextura1                   = texture       {
      pigment             {
        crackle
        translate         5
        scale             0.1
        pigment_map       {
          [ 0.0 Gray40 ]
          [ 1.0 Black  ]
        }
      }
      normal              {
        crackle
        translate         5
        scale             0.1
      }
    }
    #local MyTextura2   = texture       {
      pigment       {
        Gray30
      }
      normal   {
        radial
        ramp_wave
        frequency       60
        rotate  90*x
      }
    }
    
    gradient  z
    ramp_wave
    scale       2
    translate   -0.5*z
    texture_map       {  
      [ 0.0  MyTextura1   ]
      [ 0.5  MyTextura1   ]
      [ 0.5  MyTextura2   ]
      [ 1.0  MyTextura2   ]
    }
  #else
    #local MyTextura1                   = texture       {
      pigment             {
        crackle
        solid
        translate         5
        scale             0.1
        pigment_map       {
          [ 0.0 Gray40 ]
          [ 1.0 Black  ]
        }
      }
      normal              {
        crackle
        solid
        translate         5
        scale             0.1
      }
    }
    #local MyTextura2   = texture       {
      pigment       {
        Gray30
      }
      normal   {
        radial      
        frequency       60
        rotate  90*x
      }
    }
    
    gradient  z
    ramp_wave 
    scale       2
    translate   -0.5*z
    texture_map       {  
      [ 0.0 MyTextura1]
      [ 0.5  MyTextura1   ]
      [ 0.5  MyTextura2   ]
      [ 1.0 MyTextura2 ]
    }
    
  #end
}

#declare RBSA_TexturaRuedaInterior      = texture       {   
    pigment       {
      White
    }      
}

#declare RBSA_TexturaMetalica                   = texture       {
  T_Chrome_3C
  normal      {
    bumps     1
    scale     0.3
  }
}

#declare RBSA_MaterialLuz                       = material      { 
  #if ( RBSA_POV3_1 )
    texture               {
      Yellow_Glass
    }
  #else    
    M_Yellow_Glass
  #end
}

#declare RBSA_DefaultTexturaRodante             = texture       {
  RBSA_TexturaRodante
}

#declare RBSA_DefaultTexturaRuedaInterior       = texture       {
  RBSA_TexturaRuedaInterior
}

#declare RBSA_DefaultTexturaMetalica            = texture       {
  RBSA_TexturaMetalica
}

#declare RBSA_DefaultMaterialLuz                = material      {
  RBSA_MaterialLuz
}

#macro RBSA_RuedaBicicletaSinAccesorios
        ( RadioTotal, DiametroRueda, AnchoMetalico,
                TexturaRodante, TexturaRuedaInterior, TexturaMetalica,
                MaterialLuz )
  #local L                      = RBSA_L;
  union                         {
    //  Rodante
    difference                  {
      torus                     {
        RadioTotal-DiametroRueda/2      , DiametroRueda/2
        rotate          90*x
      }
      cylinder                  {
        -DiametroRueda*z        , DiametroRueda*z       ,
        RadioTotal-DiametroRueda/2
      }
      texture                   {
        TexturaRodante
        scale           < RadioTotal, RadioTotal, DiametroRueda/2 >
      }
    }
    difference                  {
      torus                     {
        RadioTotal-DiametroRueda/2      , DiametroRueda/2
        rotate          90*x
      }
      difference                {
        cylinder                  {
          -0.9*DiametroRueda*z    , 0.9*DiametroRueda*z       ,
          RadioTotal+DiametroRueda/2
        }
        cylinder                  {
          -DiametroRueda*z        , DiametroRueda*z       ,
          RadioTotal-DiametroRueda/2
        }
      }
      texture                   {
        TexturaRuedaInterior
        scale           < RadioTotal, RadioTotal, DiametroRueda/2 >
      }
    }
    // SoporteMetalico
    union               {
      #local RadioMetalico      = 0.809*RadioTotal;
      #local GruesoMetalico     = 0.611*DiametroRueda;
      sor               {
        6       ,
        < RadioTotal    ,-0.7*GruesoMetalico >,
        < RadioTotal-DiametroRueda/2    ,-0.6*GruesoMetalico >,
        < RadioMetalico,-GruesoMetalico/2 >,
        < RadioMetalico, GruesoMetalico/2 >,
        < RadioTotal-DiametroRueda/2    , 0.6*GruesoMetalico >,
        < RadioTotal    , 0.7*GruesoMetalico >
        open
        rotate          90*x
      }
      #local FactorInterno      = 0.185;
      #local FactorBarrita      = 0.5*FactorInterno;
      #local BarritaMetalica    = 1.722*AnchoMetalico;
      #local FactorDisco        = 0.537;
      cylinder          {
        -BarritaMetalica/2*z      , +BarritaMetalica/2*z    ,
        FactorBarrita*AnchoMetalico/2
      }
      cylinder          {
        -AnchoMetalico/2*z      , +AnchoMetalico/2*z    ,
        FactorInterno*AnchoMetalico/2
      }      
      disc              {
        -AnchoMetalico/2*z      , z     ,
        FactorDisco*AnchoMetalico/2
      }
      disc              {
        +AnchoMetalico/2*z      , z     ,
        FactorDisco*AnchoMetalico/2
      }
      // soporte tornillo
      #local SoporteTornillo    = union         {
        torus           {
          0.55*FactorDisco*AnchoMetalico/2,
          0.10*FactorDisco*AnchoMetalico/2
          rotate        90*x
          translate     AnchoMetalico/2*z
        }
        torus           {
          0.35*FactorDisco*AnchoMetalico/2,
          0.15*FactorDisco*AnchoMetalico/2
          rotate        90*x
          translate     AnchoMetalico/2*z
        }
        cylinder        {
          -0.10*FactorDisco*AnchoMetalico/2*z,
          0.10*FactorDisco*AnchoMetalico/2*z,
          0.55*FactorDisco*AnchoMetalico/2
          translate     AnchoMetalico/2*z
        }
        prism           {
          0     , 0.40*FactorDisco*AnchoMetalico/2, 7
          #local PuntoPrisma    = 0.30*FactorDisco*AnchoMetalico/2*x;
          #local C              = 0;
          #while ( C < 7 )
            #local PuntoPrisma  = vaxis_rotate( PuntoPrisma, y, 60 );
            < PuntoPrisma.x, PuntoPrisma.z >
            #local C            = C+1;
          #end
          rotate        90*x
          translate     AnchoMetalico/2*z
        }
      }
      object            {
        SoporteTornillo
      }
      object            {
        SoporteTornillo
        scale           < 1, 1, -1 >
      }
      #local C                  = 0;
      #local TotalGrupoBarrita  = 7;
      #local RadioBarrita       = 0.025*AnchoMetalico;
      #local PuntoBarrita       =
                0.75*FactorDisco*AnchoMetalico/2*y+AnchoMetalico/2*z;
      #local FinalBarrita       = RadioMetalico*x;
      #local FinalBarrita       =
                vaxis_rotate( FinalBarrita, z, 360/( 4*TotalGrupoBarrita ) );
      #local BarraGuia          = union         {
        cylinder                {
          PuntoBarrita        , FinalBarrita  ,
          RadioBarrita
        }
        cylinder                {
          PuntoBarrita+0.90*( FinalBarrita-PuntoBarrita )       ,
          FinalBarrita          ,
          2*RadioBarrita
        }
      }
      #while ( C < TotalGrupoBarrita )
        union           {
          object                {
            BarraGuia
          }
          object        {
            BarraGuia
            scale                 < 1, -1, 1 >
          }
          object        {
            BarraGuia
            scale                 < 1, 1, -1 >
            rotate                360/( 4*TotalGrupoBarrita )*z
          }
          object        {
            BarraGuia
            scale                 < 1, -1, 1 >
            rotate                360/( 5*TotalGrupoBarrita )*z
          }
          rotate          C*360/( TotalGrupoBarrita )*z
        }
        #local C                = C+1;
      #end
      texture           {
        TexturaMetalica
      }
    }
    // Luz
    #if ( RBSA_LuzBicicleta )
      prism             {
        -DiametroRueda/4        , +DiametroRueda/4      , 7
        < ( RadioTotal-2.0*DiametroRueda ).x, 0 >
        #local Punto1   =
           vaxis_rotate( ( RadioTotal-1.8*DiametroRueda )*x, z, 360/25 );
        < Punto1.x, Punto1.y >   
        #local Punto1   =
           vaxis_rotate( ( RadioTotal-1.5*DiametroRueda )*x, z, 360/25 );
        < Punto1.x, Punto1.y >              
        < ( RadioTotal-1.5*DiametroRueda ).x, 0 >
        #local Punto1   =
           vaxis_rotate( ( RadioTotal-1.5*DiametroRueda )*x, z,-360/25 );
        < Punto1.x, Punto1.y >              
        #local Punto1   =
           vaxis_rotate( ( RadioTotal-1.8*DiametroRueda )*x, z,-360/25 );
        < Punto1.x, Punto1.y >   
        < ( RadioTotal-2.0*DiametroRueda ).x, 0 >
        rotate          90*x
        material        {
          MaterialLuz
        }
      }
      cylinder          {
        ( RadioTotal-1.75*DiametroRueda )*x-DiametroRueda/3.9*z   ,
        ( RadioTotal-1.75*DiametroRueda )*x+DiametroRueda/3.9*z   ,
        DiametroRueda/10
        texture         {
          TexturaMetalica
        }
      }
    #end
  }
#end

#macro RBSA_RuedaBicicletaPequena()
  #local L                      = RBSA_L;
  #local RadioTotal             = 215*L;
  #local DiametroRueda          = 36*L;
  #local AnchoMetalico          = 54*L;
  object        {
    RBSA_RuedaBicicletaSinAccesorios
      ( RadioTotal, DiametroRueda, AnchoMetalico,
        RBSA_DefaultTexturaRodante, RBSA_DefaultTexturaRuedaInterior,
        RBSA_DefaultTexturaMetalica, RBSA_DefaultMaterialLuz )
  }
#end
