// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com
// Apdo. 964-1250 Escaz, San Jos, Costa Rica
// dedicado a Jess y Mara...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 15 febrero 2011
//                
// Mis dibujos estn en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY 
//
// STMC_Aux.pov es un archivo experimental en desarrollo
// Se recomienda no utilizarlo directamente en sus propias
// animaciones, y esperar futuras versiones. 
// Se le conocen algunos errores.
// Se incluye aqu como auxiliar
// para la graficacion del objeto STMC_Silla
//
// #macro STMC_Determinante ( Det, A11, A12, A21, A22 )
//      Calcula el determinante de una matriz 2*2
//                     A11   A12   = A11*A12-A12*A21
//                     A21   A22  
//      Det     : OUTPUT resultado float del determinante
//      A11     : casilla A11 de la matriz
//      A12     : casilla A12 de la matriz
//      A21     : casilla A21 de la matriz
//      A22     : casilla A22 de la matriz
//
// #macro STMC_SistemaDoble ( Solucion, A, B, C, D, E, F )
//      Resuelve el sistema de ecuaciones lineales 2*2
//              Ax+By   = C     Solucion = < x,y,Determinante  >
//              Dx+Ey   = F
//      Solucion        : OUTPUT devuelve la solucion como <x,y,Determinante>
//              O podria ser vector nulo si Determinante = 0
//      A               : constante multiplica x en ecuacion primera
//      B               : constante multiplica y en ecuacion primera
//      C               : constante resultado en ecuacion primera
//      D               : constante multiplica x en ecuacion segunda
//      E               : constante multiplica y en ecuacion segunda
//      F               : constante resultado en ecuacion segunda
//
// #macro STMC_CentroTriplePunto( Centro, C, P, Q )
//      Calculo Centro de un circulo que pase por los vectores C, P y Q
//      o vector nulo si fracaso
//      STMC_BanderaCentro      = true  si exito
//                                false si fracaso
//      Centro          : OUTPUT devuelve el centro solicitado o vector
//              nulo si fracaso
//      C               : Punto del crculo
//              Se recomienda (innecesario) colocarlo en medio de P y Q.
//      P               : Otro punto del crculo
//      Q               : Otro punto del crculo
//
// #macro STMC_CentroEsfericoTriplePunto
//        ( CentroEsferico, CentroTriangulo, C, P, Q, RadioEsfera )
//      Calculo Centro de un circulo y esfera
//      que pase por los vectores C, P y Q
//      o vector nulo si fracaso, la esfera tiene radio RadioEsfera
//      STMC_BanderaCentro      = true  si exito en triangulo
//                                false si fracaso en triangulo
//      STMC_BanderaEsfera      = true  si exito en esfera
//                                false si fracaso en esfera ( vector nulo )
//      Nota: es posible exito en Centro de Triangulo y fracaso en Esfera
//              por ejemplo si RadioEsfera es muy pequeno
//      CentroEsferico  : OUTPUT devuelve el centro esfera solicitado o vector
//              nulo si fracaso
//      CentroTriangulo : OUTPUT devuelve el centro plano de triangulo
//              solicitado o vector nulo si fracaso
//      C               : Punto del crculo inscrito a esfera
//      P               : Otro punto del crculo inscrito a esfera
//      Q               : Otro punto del crculo inscrito a esfera
//      RadioEsfera     : Radio solicitado de esfera
//
// #macro STMC_SweepTriplePunto( P, C, Q, Radio, BanderaEsfera )
//      Genera Torus que pase por los vectores P, C y Q
//      C en medio de P y Q, el torus de grueso Radio.
//      BanderaEsfera true agrega esferas en los extremos
//      En la versin actual el ngulo debera ser menor o igual a 180.
//              Si alineados genera cilindro.
//      P               : punto inicial del recorrido del torus
//      C               : punto central del recorrido del torus
//      Q               : punto final del recorrido del torus
//      Radio           : radio interno del torus
//      BanderaEsfera   : si true coloca esferas en los extremos
//                        si false se ignora
//
// #macro STMC_AnilloTriplePunto( P, C, Q, GruesoR, GruesoZ, BanderaFinal )
//      Similar a STMC_SweepTriplePunto (arriba), pero el torus
//      es de base interna cuadrangular. Es decir se genera
//      por diferencia de anillos cilindricos.
//      Genera Anillo Cilidrico que pase por los vectores P, C y Q
//      C en medio de P y Q.
//      GruesoR es el "grueso" del anillo cilindrico y GruesoZ es el
//      grueso perpendicular al recorrido sweep.
//      Los puntos se colocan centrados al recorrido.
//      BanderaEsfera true agrega cilindros en los extremos
//      En la versin actual el ngulo debera ser menor o igual a 180.
//              ADVERTENCIA: Si alineados genera cilindro.
//              Disculpas por los problemas que esto podria causar
//              Podria remediarlo induciendo un pequeo defecto
//              en el recorrido.
//      P               : punto inicial del recorrido del anillo cilidrico
//      C               : punto central del recorrido del anillo cilindro
//      Q               : punto final del recorrido del anillo cilindrico
//      GruesoR         : grueso radial del anillo
//      GruesoZ         : grueso perpendicular al radio del anillo
//      BanderaFinal    : si true coloca cilindros en los extremos
//                        si false se ignora
// #macro STMC_SegmentoTriplePunto( P, C, Q,
//                        GruesoZ, SegmentoSector, BanderaFinal )
//      Similar a STMC_SweepTriplePunto (arriba), pero se genera
//      un Segmento o un SectorCircular segun bandera SegmentoSector
//      el borde es plano o toroidal segun BanderaFinal
//      Genera Segmento o Sector Circular que pase por la curva P, C y Q
//      C en medio de P y Q.
//      GruesoZ es el "grueso"
//      grueso perpendicular al recorrido sweep.
//      Los puntos se colocan centrados al recorrido.
//      BanderaEsfera true redondea con toroides y cilindros
//      los bordes para que queden curvos
//      En la versin actual el ngulo debera ser menor o igual a 180.
//              ADVERTENCIA: Si alineados genera cilindro.
//              Disculpas por los problemas que esto podria causar
//              Podria remediarlo induciendo un pequeo defecto
//              en el recorrido.
//      P               : punto inicial del arco circular
//      C               : punto central del arco circular curva
//      Q               : punto final de la curva del arco circular
//      GruesoZ         : grueso perpendicular al radio del anillo
//      SegmentoSector  : si true se trata de Segmento Circular
//                              relleno hasta cuerda interior
//                        si false se trata de Sector Circular
//                              relleno hasta centro del circulo
//      BanderaFinal    : si false son bordes planos
//                        si true los bordes son curvos con toriodes, etc
//                        si false se ignora
//
// #macro STMC_InterpolacionTriplePunto( Resultado, C, P, Q, FraccionU )
//      Calculo Interpolacion de un arco de circulo que pasa
//      por los vectores C, P y Q.  C en el centro de curva
//      o vector nulo si fracaso
//      STMC_BanderaCentro      = true  si exito
//                                false si fracaso
//      El resultado es algo asi a P+FraccionU*(Q-P), pero en la "curva"
//      Resultado       : OUTPUT devuelve el vector interpolado solicitado o
//              nulo si fracaso
//      C               : Punto del crculo
//              Se deberia colocar en medio de P y Q.
//      P               : Otro punto del crculo
//      Q               : Otro punto del crculo
//      FraccionU       : Recorre de 0 a 1.  0 es P y 1 es Q.
//
// Aclaracion:
// Las macros STMC_Reorient_Trans y STMC_Axis_Rotate_Trans
// se transcribe casi identica a la original
// en "transforms.inc" Reorient_Trans y Axis_Rotate_Trans
// se copia por compatibilidad con POVRay 3.1.
// La macro STMC_VPerp_To_Vector
// se tomo de "math.inc" VPerp_To_Vector
// por compatibilidad con POVRay 3.1.
//

#if ( version < 3.2 )
  #ifndef( STMC_POV3_1 )
    #declare STMC_POV3_1        = on;
  #end
#else
  #ifndef( STMC_POV3_1 )
    #declare STMC_POV3_1        = off;
  #end
#end

#if ( STMC_POV3_1 )
// Las macros STMC_Reorient_Trans y STMC_Axis_Rotate_Trans
// se transcribe casi identica a la original
// en "transforms.inc" Reorient_Trans y Axis_Rotate_Trans
// se copia por compatibilidad con POVRay 3.1.
// La macro STMC_VPerp_To_Vector
// se tomo de "math.inc" VPerp_To_Vector
// por compatibilidad con POVRay 3.1.


// Returns a vector perpendicular to V
// Author: Tor Olav Kristensen
#macro STMC_VPerp_To_Vector(v0)
   #if (vlength(v0) = 0)
      #local vN = <0, 0, 0>;
   #else
      #local Dm = min(abs(v0.x), abs(v0.y), abs(v0.z));
      #if (abs(v0.z) = Dm)
         #local vN = vnormalize(vcross(v0, z));
      #else
         #if (abs(v0.y) = Dm)
            #local vN = vnormalize(vcross(v0, y));
         #else
            #local vN = vnormalize(vcross(v0, x));
         #end
      #end
   #end
   vN
#end

// rotate around a specific axis
// Author: Rune S. Johansen
#macro STMC_Axis_Rotate_Trans(Axis, Angle)
   #local vX = vaxis_rotate(x,Axis,Angle);
   #local vY = vaxis_rotate(y,Axis,Angle);
   #local vZ = vaxis_rotate(z,Axis,Angle);
//   transform {
      matrix < vX.x,vX.y,vX.z, vY.x,vY.y,vY.z, vZ.x,vZ.y,vZ.z, 0,0,0 >
//   }
#end


// based on original Reorient() macro by John VanSickle
#macro STMC_Reorient_Trans(Axis1, Axis2)
   #local vX1 = vnormalize(Axis1);
   #local vX2 = vnormalize(Axis2);
   #local Y = vcross(vX1, vX2);
   #if(vlength(Y) > 0)
      #local vY = vnormalize(Y);
      #local vZ1 = vnormalize(vcross(vX1, vY));
      #local vZ2 = vnormalize(vcross(vX2, vY));
//      transform {
         matrix < vX1.x, vY.x,vZ1.x, vX1.y,vY.y,vZ1.y, vX1.z,vY.z, vZ1.z, 0,0,0 >
         matrix < vX2.x,vX2.y,vX2.z,  vY.x,vY.y, vY.z, vZ2.x,vZ2.y,vZ2.z, 0,0,0 >
//      }
   #else
      #if (vlength(vX1-vX2)=0)
         transform {}
      #else
         #local vZ = STMC_VPerp_To_Vector(vX2);
//         transform {
         STMC_Axis_Rotate_Trans(vZ,180)
//}
      #end
   #end
#end

#else
  #include "transforms.inc"
#end

#macro STMC_Determinante ( Det, A11, A12, A21, A22 )
  #local Det            = ( A11*A22-A12*A21 );
#end

#macro STMC_SistemaDoble ( Solucion, A, B, C, D, E, F )
  #local Det            = 0;
  STMC_Determinante( Det, A, B, D, E )
  #if ( Det = 0 )
    #local Solucion   = < 0, 0, 0 >;
  #else
    #local DetX         = 0;
    #local DetY         = 0;
    STMC_Determinante( DetX, C, B, F, E )
    STMC_Determinante( DetY, A, C, D, F )
    #local Solucion   = < DetX/Det, DetY/Det, Det >;
  #end
#end

// ( STMC_BanderaCentro = true ), confirma exito
#macro STMC_CentroTriplePunto( Centro, C, P, Q )
  #local CP             = P-C;
  #local CQ             = Q-C;
  #local CP2            = ( C+P )/2;
  #local CQ2            = ( C+Q )/2;
  #local AEQ            = vdot( CQ, CQ );
  #local BEQ            = vdot( CQ, CP );
  #local CEQ            = vdot( CQ2, CQ );
  #local DEQ            = BEQ;
  #local EEQ            = vdot( CP, CP );
  #local FEQ            = vdot( CP2, CP );
  #local STMC_Solucion  = <0,0,0>;
  STMC_SistemaDoble( STMC_Solucion, AEQ, BEQ, CEQ, DEQ, EEQ, FEQ )
  #if ( STMC_Solucion.z = 0 )
    #local Centro       = < 0, 0, 0 >;
    #declare STMC_BanderaCentro = false;
  #else
    #local Centro       = ( STMC_Solucion.x*CQ+STMC_Solucion.y*CP );
    #local Normal       = vnormalize( vcross( CP, CQ ) );
    #local Centro       = Centro+vdot( C, Normal )*Normal;
    #declare STMC_BanderaCentro = true;
  #end
#end

#macro STMC_InterpolacionTriplePunto( Resultado, C, P, Q, FraccionU )
  #local Centro         = < 0, 0, 0 >;
  STMC_CentroTriplePunto( Centro, C, P, Q )
  #if ( STMC_BanderaCentro )
    #local CP         = vnormalize( P-Centro );
    #local CQ         = vnormalize( Q-Centro );
    #local Normal     = vcross( CP, CQ );
    #local Angulo     = FraccionU*degrees( acos( vdot( CP, CQ ) ) );
    #local Resultado  = vaxis_rotate( P-Centro, Normal, Angulo )+Centro;
  #else
    #local Resultado  = P+FraccionU*( Q-P );
  #end
#end

#macro STMC_CentroEsfericoTriplePunto
        ( CentroEsferico, CentroTriangulo, C, P, Q, RadioEsfera )
  STMC_CentroTriplePunto( CentroTriangulo, C, P, Q )
  #if ( STMC_BanderaCentro )
    #local Normal       = vnormalize( vcross( P-C, Q-C ) );
    #local PivReal      =
        RadioEsfera*RadioEsfera-vdot( P-CentroTriangulo, P-CentroTriangulo );
    #if ( PivReal <= 0 )
      #declare STMC_BanderaEsfera       = false;
      #local CentroEsferico             = <0,0,0>;
    #else
      #declare STMC_BanderaEsfera       = true;
      #local CentroEsferico             =
                        CentroTriangulo+sqrt( PivReal )*Normal;
    #end
  #end
#end

#macro STMC_SweepTriplePunto( P, C, Q, Radio, BanderaEsfera )
  #local Centro         = < 0, 0, 0 >;
  STMC_CentroTriplePunto( Centro, C, P, Q )
  union                 {
  #if ( STMC_BanderaCentro )
    difference            {
      #local RadioPr    = sqrt( vdot( P-Centro, P-Centro ) );
      #local CP         = vnormalize( P-Centro );
      #local CQ         = vnormalize( Q-Centro );
      #local Normal     = vcross( CP, CQ );
      #local Angulo     = degrees( acos( vdot( CP, CQ ) ) );

      torus               {
        RadioPr   , Radio
        sturm
        #if ( STMC_POV3_1 )
          STMC_Reorient_Trans( y, Normal )
        #else
          Reorient_Trans( y, Normal )
        #end
      }
      plane               {
        #local Normal1  = vnormalize( vcross( CP, Normal ) );
        -Normal1         , 0
      }
      plane               {
        #local Normal1  = vnormalize( vcross( Normal, CQ ) );
        -Normal1         , 0
      }
      translate         Centro
    }
  #else
      cylinder          {
        P       , Q     , Radio
      }
  #end
  #if ( BanderaEsfera )
    sphere      {
      P         , Radio
    }
    sphere      {
      Q         , Radio
    }
  #end
  }     // union total
#end
// Se genera un warning, si BanderaEsfera false...

#macro STMC_AnilloTriplePunto( P, C, Q, GruesoR, GruesoZ, BanderaFinal )
  #local Centro         = < 0, 0, 0 >;
  STMC_CentroTriplePunto( Centro, C, P, Q )
  union                 {
  #if ( STMC_BanderaCentro )
    difference            {
      #local RadioPr    = sqrt( vdot( P-Centro, P-Centro ) );
      #local CP         = vnormalize( P-Centro );
      #local CQ         = vnormalize( Q-Centro );
      #local Normal     = vcross( CP, CQ );
      #local Angulo     = degrees( acos( vdot( CP, CQ ) ) );

      cylinder          {
        -GruesoZ/2*y    , +GruesoZ/2*y  , RadioPr+GruesoR/2
        #if ( STMC_POV3_1 )
          STMC_Reorient_Trans( y, Normal )
        #else
          Reorient_Trans( y, Normal )
        #end
      }
      cylinder          {
        -1.1*GruesoZ/2*y    , +1.1*GruesoZ/2*y  , RadioPr-GruesoR/2
        #if ( STMC_POV3_1 )
          STMC_Reorient_Trans( y, Normal )
        #else
          Reorient_Trans( y, Normal )
        #end
      }
      plane               {
        #local Normal1  = vnormalize( vcross( CP, Normal ) );
        -Normal1         , 0
      }
      plane               {
        #local Normal1  = vnormalize( vcross( Normal, CQ ) );
        -Normal1         , 0
      }
      translate         Centro
    }
  #else
      cylinder          {
        P       , Q     , Radio
      }
  #end
  #if ( BanderaFinal )
    #if ( STMC_BanderaCentro )
      cylinder          {
        -GruesoZ/2*y    , GruesoZ/2*y   , GruesoR/2
        #if ( STMC_POV3_1 )
          STMC_Reorient_Trans( y, Normal )
        #else
          Reorient_Trans( y, Normal )
        #end
        translate       P
      }
      cylinder          {
        -GruesoZ/2*y    , GruesoZ/2*y   , GruesoR/2
        #if ( STMC_POV3_1 )
          STMC_Reorient_Trans( y, Normal )
        #else
          Reorient_Trans( y, Normal )
        #end
        translate       Q
      }
    #else
      sphere      {
        P         , Radio
      }
      sphere      {
        Q         , Radio
      }
    #end
  #end
  }     // union total
#end
// se genera un warning si BanderaFinal es false

#macro STMC_SegmentoTriplePunto( P, C, Q,
                        GruesoZ, SegmentoSector, BanderaFinal )
  #local Centro         = < 0, 0, 0 >;
  STMC_CentroTriplePunto( Centro, C, P, Q )
  union                 {
  #if ( STMC_BanderaCentro )
    difference            {
      #local RadioPr    = sqrt( vdot( P-Centro, P-Centro ) );
      #local CP         = vnormalize( P-Centro );
      #local CQ         = vnormalize( Q-Centro );
      #local PQ         = vnormalize( Q-P );
      #local Normal     = vcross( CP, CQ );
      #local Angulo     = degrees( acos( vdot( CP, CQ ) ) );

      cylinder          {
        -GruesoZ/2*y    , +GruesoZ/2*y  , RadioPr
        #if ( STMC_POV3_1 )
          STMC_Reorient_Trans( y, Normal )
        #else
          Reorient_Trans( y, Normal )
        #end
      }
      #if ( SegmentoSector )
        plane               {
          #local Normal1  = vnormalize( vcross( PQ, Normal ) );
          Normal1          , vdot( Normal1, P-Centro )
        }
      #else
        plane               {
          #local Normal1  = vnormalize( vcross( CP, Normal ) );
          -Normal1         , 0
        }
        plane               {
          #local Normal1  = vnormalize( vcross( Normal, CQ ) );
          -Normal1         , 0
        }
      #end
      translate         Centro
    }
  #else
      cylinder          {
        P       , Q     , Radio
      }
  #end
  #if ( BanderaFinal )
    #if ( STMC_BanderaCentro )
      object      {
        STMC_SweepTriplePunto( P, C, Q, GruesoZ/2, true )
      }
      #if ( SegmentoSector )
        cylinder        {
          P             , Q     , GruesoZ/2
        }
      #else
        cylinder        {
          P             , Centro        , GruesoZ/2
        }
        cylinder        {
          Q             , Centro        , GruesoZ/2
        }
        sphere          {
          Centro        , GruesoZ/2
        }
      #end
    #else
      sphere      {
        P         , Radio
      }
      sphere      {
        Q         , Radio
      }
    #end
  #end
  }     // union total
#end
// se genera un warning si BanderaFinal es false


