// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 19 abril 2011
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY 
//
// La macro RCPS_Tulipan( Semilla, TotalHojas )
// genera un objeto con
// las siguientes caracteristicas
// Base de Tulipan en origen
// orientado hacia y+
// Alto Tulipan         : 80*L en y (aleatorio)
// Radio Tulipan        : 50*L (aleatorio)
// Largo Tallo          : 3*AltoTulipan aprox.
// Radio Tallo          : 1.5*L
// Final Tallo          : Aleatorio en RCPS_FinalTallo
//              El vector 3D se genera automaticamente con la macro
//              Nota: -RCPS_FinalTallo.y/3 = AltoTulipan aprox.
// Los parametros serian
//      Semilla         : Numero seed para aleatoriedad
//                        misma Semilla implica misma flor
//      TotalHojas      : Numero de hojas en el tallito
//                        a partir de 0
//
// La macro RCPS_Margarita( Semilla, TotalHojas )
// genera un objeto con
// las siguientes caracteristicas
// Base de Margarita en origen
// orientado hacia z- (petalos radiales cerca del plano xy)
// Alto Margarita       : aleatorio, forma planar
// Alto Centro          : 4*L
// Radio Margarita      : 35*L aprox
// Largo Tallo          : 350*L aprox inclinado 45 grados (y-z+).
// Radio Tallo          : 1.5*L
// Final Tallo          : Aleatorio en RCPS_FinalTallo
//              El vector 3D se genera automaticamente con la macro
// Los parametros serian
//      Semilla         : Numero seed para aleatoriedad
//                        misma Semilla implica misma flor
//      TotalHojas      : Numero de hojas en el tallito
//                        a partir de 0
//
// Los objetos tambien se pueden generar via una macro principal
// #macro   RCPS_FlorPlasticaColorVariado
//        ( Estilo, Semilla, TexturaPrincipal,
//              TexturaSecundaria, TexturaHoja, TexturaTallo, TotalHojas  )
//  Existen dos estilos predefinidos
//      Estilo                  : RCPS_EstiloTulipan o RCPS_EstiloMargarita
//      Semilla                 : Numero seed para aleatoriedad
//      TexturaPrincipal        : Se aplica a los petalos
//      TexturaSecundaria       : Se aplica a los estambres
//      TexturaHojas            : Se aplica a las hojas
//      TexturaTallo            : Se aplica al tallito
//      TotalHojas              : A partir de 0, controla el total
//              de hojas en el tallito
//
// Las siguientes variables no deberian modificarse
// RCPS_TexturaPrincipalTulipan
//      Define la textura utilizada en los petalos del Tulipan
// RCPS_TexturaSecundariaTulipan
//      Define la textura utilizada en los estambres del Tulipan
// RCPS_TexturaHojaTulipan
//      Define la textura utilizada en las hojas del Tulipan
// RCPS_TexturaTalloTulipan
//      Define la textura utilizada en el tallo del Tulipan
// RCPS_TexturaPrincipalMargarita
//      Define la textura utilizada en los petalos de la Margarita
// RCPS_TexturaSecundariaMargarita
//      Define la textura utilizada en los estambres de la Margarita
// RCPS_TexturaHojaMargarita
//      Define la textura utilizada en las hojas de la Margarita
// RCPS_TexturaTalloMargarita
//      Define la textura utilizada en el tallo de la Margarita
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)
// #declare RCPS_POV3_1         = off;
//      Valor on activa modo de compatibilidad con POVRay 3.1.
//      En algunas versiones se activara automaticamente.
// #declare RCPS_TexturaPrincipalTulipanDefault         = texture ...
//      Define la textura utilizada en los petalos del Tulipan
// #declare RCPS_TexturaSecundariaTulipanDefault        = texture ...
//      Define la textura utilizada en los estambres del Tulipan
// #declare RCPS_TexturaHojaTulipanDefault              = texture ...
//      Define la textura utilizada en las hojas del Tulipan
// #declare RCPS_TexturaTalloTulipanDefault             = texture ...
//      Define la textura utilizada en el tallo del Tulipan
// #declare RCPS_TexturaPrincipalMargaritaDefault            = texture ...
//      Define la textura utilizada en los petalos de la Margarita
// #declare RCPS_TexturaSecundariaMargaritaDefault           = texture ...
//      Define la textura utilizada en los estambres de la Margarita
// #declare RCPS_TexturaHojaMargaritaDefault                 = texture ...
//      Define la textura utilizada en las hojas de la Margarita
// #declare RCPS_TexturaTalloMargaritaDefault                = texture ...
//      Define la textura utilizada en el tallo de la Margarita
//
// Para colocarlo simplemente
// #include "RCPS.inc"
// object { RCPS_Margarita( 1232, 4 ) }
//
// Otro ejemplo con modificador
// #include "RCPS.inc"
// #declare RCPS_TexturaPrincipalMargaritaDefault    = texture       {
//   average
//   texture_map        {
//     [ 0.8 RCPS_TexturaPrincipalMargarita    ]
//     [ 0.2 RCPS_TexturaHojaMargarita         ]
//   }
// }
// object       { RCPS_Margarita( 1232, 4 ) }
//

#include "TMPS_AUX.inc"
#include "colors.inc"

#if ( version < 3.2 )
  #ifndef( TMPS_POV3_1 )
    #declare TMPS_POV3_1        = on;
  #end
#else
  #ifndef( TMPS_POV3_1 )
    #declare TMPS_POV3_1        = off;
  #end
#end

#declare TMPS_L                 = 0.001;

#declare TMPS_EstiloTulipan       = 1;
#declare TMPS_EstiloMargarita     = 4;

#declare TMPS_TexturaPrincipalTulipan     = texture       {
  #if ( TMPS_POV3_1 )
    pigment     {
      gradient  y
      rotate    25*x+30*y   
      poly_wave 4        
      scale     0.055*y
      turbulence        0.2
      pigment_map       {
        [ 0.0 White             ]
        [ 1.0 MediumOrchid      ]
      }
    }
  #else
    pigment     {
      gradient  y
      rotate    25*x+30*y   
      poly_wave 4        
      scale     0.055*y
      turbulence        0.2
      pigment_map       {
        [ 0.0 White             ]
        [ 1.0 MediumOrchid      ]
      }
    } 
    normal      {
      average
      normal_map        {
        [ 0.4 facets                    ]
        [ 0.6 agate  scale      0.1     ]
      }
    }
  #end
}

#declare TMPS_TexturaSecundariaTulipan    = texture       {
  #if ( TMPS_POV3_1 )
    pigment       { Yellow }
  #else
    pigment     { 
      cells
      rotate    20
      scale     0.001
      turbulence        2
      pigment_map       {
        [ 0.0 Yellow ]
        [ 1.0 BrightGold  ]
      }
    }
    normal      {
      cells
      rotate    20
      scale     0.001
    }
  #end
}

#declare TMPS_TexturaHojaTulipan          = 
    texture     {
      pigment     {
        checker 
        pigment   {
          gradient        z 
          turbulence      0.3 
          rotate          30*y
          sine_wave
          scale           0.001
          pigment_map     {
            [ 0.0 LimeGreen ]
            [ 1.0 MediumForestGreen ]
          }
        }
        pigment   {
          gradient        z
          turbulence      0.3
          rotate          -30*y
          sine_wave
          scale           0.002
          pigment_map     {
            [ 0.0 LimeGreen ]
            [ 1.0 MediumForestGreen ]
          }
        }
      }   
      normal            {
        checker
        normal   {
          gradient        z 
          turbulence      0.3 
          rotate          30*y
          sine_wave
          scale           0.001
        }
        normal   {
          gradient        z 
          turbulence      0.3 
          rotate          -30*y
          sine_wave
          scale           0.001
        }
      }  
    }
    texture     {
      pigment     {
        gradient          x
        turbulence        0.25
        scallop_wave
        scale             0.02
        pigment_map       {
          [ 0.0 ForestGreen  ]
          [ 1.0 rgbt 1       ]
        }
      } 
      normal    {
        gradient          x
        translate         0.5*x
        turbulence        0.25
        sine_wave
        scale             0.01
      }
    }


#declare TMPS_TexturaTalloTulipan         = texture       { 
    pigment     {
      gradient  x  
      turbulence        0.5
      scallop_wave
      scale     0.003
      rotate    10*z
      pigment_map       {
        [ 0.0 LimeGreen         ]
        [ 1.0 MediumForestGreen ]
      } 
    }
}

#ifndef ( TMPS_TexturaPrincipalTulipanDefault )
  #declare TMPS_TexturaPrincipalTulipanDefault            = texture       {
    TMPS_TexturaPrincipalTulipan
  }
#end

#ifndef ( TMPS_TexturaSecundariaTulipanDefault )
  #declare TMPS_TexturaSecundariaTulipanDefault           = texture       {
    TMPS_TexturaSecundariaTulipan
  }
#end

#ifndef ( TMPS_TexturaHojaTulipanDefault )
  #declare TMPS_TexturaHojaTulipanDefault                 = texture       {
    TMPS_TexturaHojaTulipan
  }
#end

#ifndef ( TMPS_TexturaTalloTulipanDefault )
  #declare TMPS_TexturaTalloTulipanDefault                = texture       {
    TMPS_TexturaTalloTulipan
  }
#end

#declare TMPS_TexturaPrincipalMargarita     = texture       {
    pigment     { 
      onion
      turbulence        0.5
      scale     0.005
      pigment_map       {
        [ 0.0 rgb 0.80  ]
        [ 1.0     White ]
      }
    }
    normal      {
      spotted
      turbulence   1.0  
      scale     0.001
    }
}

#declare TMPS_TexturaSecundariaMargarita    = texture       {
    #if ( TMPS_POV3_1 )
      pigment       { Yellow  }
    #else
    pigment     { 
      cells
      rotate    20
      scale     0.001
      turbulence        2
      pigment_map       {
        [ 0.0 Yellow ]
        [ 1.0 Coral  ]
      }
    }
    normal      {
      cells
      rotate    20
      scale     0.001
    }
  #end
}

#declare TMPS_TexturaHojaMargarita          = 
    texture     {
      pigment     {
        checker 
        pigment   {
          gradient        z 
          turbulence      0.3 
          rotate          30*y
          sine_wave
          scale           0.001
          pigment_map     {
            [ 0.0 LimeGreen ]
            [ 1.0 MediumForestGreen ]
          }
        }
        pigment   {
          gradient        z
          turbulence      0.3
          rotate          -30*y
          sine_wave
          scale           0.002
          pigment_map     {
            [ 0.0 LimeGreen ]
            [ 1.0 MediumForestGreen ]
          }
        }
      }   
      normal            {
        checker
        normal   {
          gradient        z 
          turbulence      0.3 
          rotate          30*y
          sine_wave
          scale           0.001
        }
        normal   {
          gradient        z 
          turbulence      0.3 
          rotate          -30*y
          sine_wave
          scale           0.001
        }
      }  
    }
    texture     {
      pigment     {
        gradient          x
        turbulence        0.25
        scallop_wave
        scale             0.02
        pigment_map       {
          [ 0.0 ForestGreen  ]
          [ 1.0 rgbt 1       ]
        }
      } 
      normal    {
        gradient          x
        translate         0.5*x
        turbulence        0.25
        sine_wave
        scale             0.01
      }
    }

#declare TMPS_TexturaTalloMargarita         = texture       {
    pigment     {
      gradient  x  
      turbulence        0.5
      scallop_wave
      scale     0.003
      rotate    10*z
      pigment_map       {
        [ 0.0 LimeGreen         ]
        [ 1.0 MediumForestGreen ]
      } 
    }
}

#ifndef ( TMPS_TexturaPrincipalMargaritaDefault )
  #declare TMPS_TexturaPrincipalMargaritaDefault            = texture       {
    TMPS_TexturaPrincipalMargarita
  }
#end

#ifndef ( TMPS_TexturaSecundariaMargaritaDefault )
  #declare TMPS_TexturaSecundariaMargaritaDefault           = texture       {
    TMPS_TexturaSecundariaMargarita
  }
#end

#ifndef ( TMPS_TexturaHojaMargaritaDefault )
  #declare TMPS_TexturaHojaMargaritaDefault                 = texture       {
    TMPS_TexturaHojaMargarita
  }
#end

#ifndef ( TMPS_TexturaTalloMargaritaDefault )
  #declare TMPS_TexturaTalloMargaritaDefault                = texture       {
    TMPS_TexturaTalloMargarita
  }
#end

#macro TMPS_PuntoAleatorio( NuevoPunto, Desplazado, Aleatorio )
  #local NuevoPunto     = NuevoPunto+
     Desplazado*< rand( Aleatorio ), rand( Aleatorio ), rand( Aleatorio ) >;
#end

#macro   TMPS_FlorPlasticaColorVariado
        ( Estilo, Semilla, TexturaPrincipal,
             TexturaSecundaria, TexturaHoja, TexturaTallo, TotalHojas  )
  #local Aleatorio      = seed( Semilla );                   
  #local L                              = TMPS_L;
  #switch ( Estilo )
    #case ( TMPS_EstiloTulipan )
      union             {
        #local AltoCapullo              =
                                ( 75+( -0.5+rand( Aleatorio ) )*15 )*L;
        #local HondoCapullo             =
                                ( 45+( -0.5+rand( Aleatorio ) )*20 )*L;
        #local TotalPetalosLado         = 5+( -0.5+rand( Aleatorio ) )*2;
        #local FraccionBaja             = 0.4+( -0.5+rand( Aleatorio ) )*0.2;
        #local GruesoLaminar            = 0.2*L;
        #local C                        = 0;
        #local PuntoO                   = <0, AltoCapullo, HondoCapullo>;
        #local PuntoOB                  = ( FraccionBaja+1 )/2*PuntoO;
        #local PuntoB                   =
                        FraccionBaja*<0, AltoCapullo,-HondoCapullo>;
        #local PuntoCero                = <0,0,0>;
        #local C                        = 0;
        #local PuntoControl             = FraccionBaja*PuntoOB;
        #local PuntoControl             =
                        vaxis_rotate( PuntoControl, y, 180 );
        #while ( C < ( TotalPetalosLado-1 ) )
          object        {
            #local PuntoControl2        =
         ( FraccionBaja+( C+1 )/TotalPetalosLado*( 1-FraccionBaja ) )*PuntoOB;
            #local PuntoControl2        =
        vaxis_rotate( PuntoControl2, y, 180*( 1-( C+1 )/TotalPetalosLado ) );
            TMPS_PuntoAleatorio( PuntoControl2, 0.1*HondoCapullo, Aleatorio )
            TMPS_SegmentoTrianguloEsfericoTriplePunto
                ( PuntoControl, PuntoControl2, PuntoCero, HondoCapullo,
                        GruesoLaminar, false, true, true, false )
            #local PuntoControl         =
                vaxis_rotate( PuntoControl2, y, 2 );
          }
          #local C                      = C+1;
        #end
        #local PuntoControlF1           = PuntoControl;
        #local C                        = 0;
        #local PuntoControl             = FraccionBaja*PuntoOB;
        #local PuntoControl             =
                  vaxis_rotate( PuntoControl, -y, 180 );
        #while ( C < ( TotalPetalosLado-1 ) )
          object        {
            #local PuntoControl2        =
         ( FraccionBaja+( C+1 )/TotalPetalosLado*( 1-FraccionBaja ) )*PuntoOB;
            #local PuntoControl2        =
        vaxis_rotate( PuntoControl2, -y, 180*( 1-( C+1 )/TotalPetalosLado ) );
            TMPS_PuntoAleatorio( PuntoControl2, 0.1*HondoCapullo, Aleatorio )
            TMPS_SegmentoTrianguloEsfericoTriplePunto
                ( PuntoControl2, PuntoControl, PuntoCero, HondoCapullo,
                        GruesoLaminar, false, true, true, false )
            #local PuntoControl         =
                vaxis_rotate( PuntoControl2, -y, 2 );
          }
          #local C                      = C+1;
        #end
        #local PuntoControlF2           = PuntoControl;
        #local PuntoOAleatorio          = PuntoO;
        TMPS_PuntoAleatorio( PuntoOAleatorio, 0.2*HondoCapullo, Aleatorio )
        object                  {
          TMPS_SegmentoTrianguloEsfericoTriplePunto
                ( PuntoControlF1, PuntoControlF2, PuntoCero, 1.3*HondoCapullo,
                        GruesoLaminar, false, true, true, false )
        }
        #local Pistilo          = <0,0.4*AltoCapullo,0.15*AltoCapullo>;
        #local PistiloAux       = <0,0.1*AltoCapullo,0.02*AltoCapullo>;
        #local RadioPistilo     = 1.5*L;
        #local AnguloPistiloAl  = 30;
        object                  {
          TMPS_SweepTriplePunto
          ( PuntoCero, PistiloAux, Pistilo, RadioPistilo, true )
          translate             0.2*AltoCapullo*y
          rotate        ( -0.5+rand( Aleatorio ) )*AnguloPistiloAl*y
          texture               {
            TexturaSecundaria
          }
        }
        #local RadioTallo       = 1.5*L;
        #local LargoTallo       = 3*AltoCapullo;
        #local TalloAux         = <0,-0.2*LargoTallo,0>;
        #local Tallo1           =
            LargoTallo*< 0.2*rand( Aleatorio ), -1, 0.2*rand( Aleatorio) >;
        object                  {
          TMPS_SweepTriplePunto
          ( PuntoCero, TalloAux, Tallo1, RadioTallo, true )
          translate             0.2*AltoCapullo*y
          texture               {
            TexturaTallo
          }
        }
        #declare TMPS_FinalTallo        = Tallo1+0.2*AltoCapullo*y;
        union                   {
          difference              {
            cone                  {
              PuntoCero           , 0,
              0.15*AltoCapullo*y   , 0.070*AltoCapullo
            }
            plane                 {
              -y   ,0
              rotate      +30*x
              translate   0.15*AltoCapullo*y-0.070*AltoCapullo*z
              rotate      90*y
            } 
            texture     {
              TexturaTallo
            }
          }
          #local Base1          = <0,-0.2*AltoCapullo,0>;
          #local MedioHoja1     = <0,-0.1*AltoCapullo,0.4*AltoCapullo>;
          #local MediaHojaA     = vaxis_rotate( MedioHoja1, y, -25 );
          #local MediaHojaB     = vaxis_rotate( MedioHoja1, y,  25 );
          #local HojaFinal1     = <0,-0.2*AltoCapullo,0.6*AltoCapullo>;
          #if ( TotalHojas >= 1 )
          union                 {
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( Base1, MediaHojaB, MediaHojaA, 0.5*AltoCapullo,
                        GruesoLaminar, true )
              texture   {
                TexturaHoja
              }
            }
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( HojaFinal1, MediaHojaA, MediaHojaB, 0.55*AltoCapullo,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            translate   0.2*AltoCapullo*y
            rotate      -10*x
            rotate      -10*z
            rotate      90*y
            translate   -0.2*( 1+rand( Aleatorio ) )*AltoCapullo*y      
            rotate      rand( Aleatorio )*360*y
          }    
          #end
          #if ( TotalHojas >= 2 )
          union                 {
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( Base1, MediaHojaB, MediaHojaA, 0.5*AltoCapullo,
                        GruesoLaminar, true )
              texture   {
                TexturaHoja
              }
            }
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( HojaFinal1, MediaHojaA, MediaHojaB, 0.55*AltoCapullo,
                        2.5*GruesoLaminar, true )
              texture   {
                TexturaHoja
              }
            }
            translate   0.2*AltoCapullo*y
            rotate      -40*x
            rotate      +10*z
            rotate      250*y
            translate   -0.1*( 1+rand( Aleatorio ) )*AltoCapullo*y
            rotate      rand( Aleatorio )*360*y   
          }
          texture               {
            TexturaHoja
          }
        }
        #end
        #local C        = 2;
        #while ( C < TotalHojas )
          union                 {
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( Base1, MediaHojaB, MediaHojaA, 0.5*AltoCapullo,
                        GruesoLaminar, true )
              texture   {
                TexturaHoja
              }
            }
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( HojaFinal1, MediaHojaA, MediaHojaB, 0.55*AltoCapullo,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            translate   0.2*AltoCapullo*y
            rotate      -10*x
            rotate      -10*z
            rotate      90*y
            translate   -0.4*( 0.5+rand( Aleatorio ) )*AltoCapullo*y      
            rotate      rand( Aleatorio )*360*y
          }
          #local C      = C+1;    
        #end 
        texture                 {
          TexturaPrincipal
        }
      }
    #break
    #case ( TMPS_EstiloMargarita )
      #local LargoMargarita     = 35*L;
      #local OscilacionLargo    = 0.05*LargoMargarita;
      #local RadioCentral       = 7*L;
      #local TotalPetalos       = 30;
      #local GruesoLaminar      = 0.2*L;
      #local AberturaAngular    = 360/( TotalPetalos-2 );
      #local OscilacionAngular  = 0.05*AberturaAngular;
      #local AnguloSelf         = 20;
                // AmplitudOscilacionAleatorio
      union             {
        #local PuntoCero        = <0,0,0>;
        #local C                = 0;
        #while ( C < TotalPetalos )
          union                 {
            #local PuntoMaximo          =
         < 0, LargoMargarita+( -0.5+rand( Aleatorio ) )*OscilacionLargo,
                0.05*LargoMargarita >;
            #local PuntoLateral         = 0.9*PuntoMaximo.y*y
                          +0.5*0.9*PuntoMaximo.y*radians( AberturaAngular )*x;
            #local PuntoLateralS        = PuntoLateral*<-1,1,1>;
            object      {
              TMPS_SegmentoTrianguloEsfericoTriplePunto
                ( PuntoLateralS, PuntoLateral, PuntoMaximo,
                0.9*LargoMargarita, GruesoLaminar, true, false, false, false )
            }
            #local PuntoLateral         = 0.9*PuntoMaximo.y*y
                          +0.60*0.9*PuntoMaximo.y*radians( AberturaAngular )*x;
            #local PuntoLateralS        = PuntoLateral*<-1,1,1>;
            object      {
              TMPS_SegmentoTrianguloEsfericoTriplePunto
                ( PuntoLateral, PuntoLateralS, PuntoCero,
                0.9*LargoMargarita, GruesoLaminar, true, true, true, false )
            }
            rotate
        C*( AberturaAngular+( -0.5+rand( Aleatorio )*OscilacionAngular ) )*z
            rotate
                AnguloSelf*< rand( Aleatorio ), rand( Aleatorio ), 0 >
          }
          #local C              = C+1;
        #end
        sphere          {
          0             , RadioCentral
          texture       {
            TexturaSecundaria
          }
          scale         <1,1,0.2>
          translate     -2*RadioCentral/5*z
        }
        union           {
           difference    {
            sphere          {
              0             , 1.1*RadioCentral
              scale         <1,1,0.2>
              translate     -1*RadioCentral/5*z
            }
            plane       {
              -z        , 0.0
            }    
            texture     {
              TexturaTallo
            }
          }
          #local RadioTallo       = 1.5*L;
          #local LargoTallo       = 10*LargoMargarita;
          #local TalloAux         = <0,-0.2*LargoTallo,0>;
          #local Tallo1           =
            LargoTallo*< 0.2*rand( Aleatorio ), -1, 0.2*rand( Aleatorio) >;
          object                  {
            TMPS_SweepTriplePunto
            ( PuntoCero, TalloAux, Tallo1, RadioTallo, true )
            rotate      -45*x
          }
          #declare TMPS_FinalTallo      = vaxis_rotate( Tallo1, x, -45 );
          cone                  {
            0  , 0,
            0.50*LargoMargarita*y   , 0.10*LargoMargarita
            translate   -0.50*LargoMargarita*y
            rotate      -45*x
          }
          texture       {
            TexturaTallo
          }
        }
        union           {
          #local Base1          = <0,-0.2*LargoMargarita,0>;
          #local MedioHoja1     = <0,-0.0*LargoMargarita,0.3*LargoMargarita*3>;
          #local MediaHojaA     = vaxis_rotate( MedioHoja1, y, -10 );
          #local MediaHojaB     = vaxis_rotate( MedioHoja1, y,  10 );
          #local HojaFinal1     = <0,-0.2*LargoMargarita,0.6*LargoMargarita*3>;
          #if ( TotalHojas >= 1 )
          union                 {
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( Base1, MediaHojaB, MediaHojaA, 2*LargoMargarita,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( HojaFinal1, MediaHojaA, MediaHojaB, 2*LargoMargarita,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            translate   0.2*LargoMargarita*y
            rotate      -10*x
            rotate      -10*z
            rotate      90*y
            translate   -1.0*( 1+rand( Aleatorio ) )*LargoMargarita*y
            rotate      360*rand( Aleatorio )*y
          }
          #end
          #if ( TotalHojas >= 2 )
          union                 {
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( Base1, MediaHojaB, MediaHojaA, 2*LargoMargarita,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( HojaFinal1, MediaHojaA, MediaHojaB, 2*LargoMargarita,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            translate   0.2*LargoMargarita*y
            rotate      -40*x
            rotate      +10*z
            rotate      250*y
            translate   -2.0*( 1+rand( Aleatorio ) )*LargoMargarita*y
            rotate      360*rand( Aleatorio )*y
          }
          #end
          #local C      = 2;
          #while ( C < TotalHojas )
          union                 {
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( Base1, MediaHojaB, MediaHojaA, 2*LargoMargarita,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            object              {
              TMPS_TrianguloEsfericoTriplePunto
                ( HojaFinal1, MediaHojaA, MediaHojaB, 2*LargoMargarita,
                        2.5*GruesoLaminar, true )
              texture           {
                TexturaHoja
              }
            }
            translate   0.2*LargoMargarita*y
            rotate      -10*x
            rotate      -10*z
            rotate      90*y
            translate   -2.0*( 0.5+rand( Aleatorio ) )*LargoMargarita*y
            rotate      360*rand( Aleatorio )*y
          }    
            #local C    = C+1;
          #end
          rotate                -45*x
          texture               {
            TexturaHoja
          }
        }

        texture         {
          TexturaPrincipal
        }
      }
    #break
  #end
#end

#macro TMPS_Tulipan( Semilla, TotalHojas )
  object        {
    TMPS_FlorPlasticaColorVariado( TMPS_EstiloTulipan, Semilla,
        TMPS_TexturaPrincipalTulipanDefault,
        TMPS_TexturaSecundariaTulipanDefault,
        TMPS_TexturaHojaTulipanDefault,
        TMPS_TexturaTalloTulipanDefault, TotalHojas )
  }
#end

#macro TMPS_Margarita( Semilla, TotalHojas )
  object        {
    TMPS_FlorPlasticaColorVariado( TMPS_EstiloMargarita, Semilla,
        TMPS_TexturaPrincipalMargaritaDefault,
        TMPS_TexturaSecundariaMargaritaDefault,
        TMPS_TexturaHojaMargaritaDefault,
        TMPS_TexturaTalloMargaritaDefault, TotalHojas )
  }
#end

